from beautiful_date.beautiful_date import BeautifulDate, \
    D, MDY, DMY, \
    M, Jan, Feb, Mar, Apr, May, Jun, Jul, Aug, Sept, Oct, Nov, Dec

from beautiful_date.beautiful_timedelta import \
    years, months, weeks, days, hours, minutes, seconds, microseconds, leapday, \
    year, month, day, hour, minute, second, microsecond, yearday, nlyearday, \
    MO, TU, WE, TH, FR, SA, SU

from beautiful_date.date_range import drange



from dateutil.relativedelta import relativedelta, weekdays
from datetime import date, datetime

from beautiful_date import BeautifulDate


class _RelativeDelta(relativedelta):
    """Same as relativedelta, but returns BeautifulDate in the result.

    Examples:
        >>> 16/Jan/2005 + 5*days
        BeautifulDate(2005, 1, 21)
    """

    def __add__(self, d):
        new_date = super().__add__(d)
        if isinstance(new_date, date) and not isinstance(new_date, datetime):
            return BeautifulDate(new_date.year, new_date.month, new_date.day)
        else:
            return new_date

    __radd__ = __add__


class BeautifulTimedelta:
    """Creates timedelta with specified time unit using operator '*'

    Examples:
        >>> 3*years
        _RelativeDelta(years=+3)

        >>> -5*weeks
        _RelativeDelta(days=-35)
    """

    def __init__(self, name):
        self.name = name

    def __rmul__(self, n):
        return _RelativeDelta(**{self.name: n})


_ = BeautifulTimedelta

years = _('years')
months = _('months')
weeks = _('weeks')
days = _('days')
hours = _('hours')
minutes = _('minutes')
seconds = _('seconds')
microseconds = _('microseconds')
leapdays = _('leapdays')
leapday = 1 * leapdays

year = _('year')
month = _('month')
day = _('day')
hour = _('hour')
minute = _('minute')
second = _('second')
microsecond = _('microsecond')

yearday = _('yearday')
nlyearday = _('nlyearday')

_weekday = _('weekday')


class BeautifulWeekday:
    """

    Examples:
        Get next Monday:
        >>> d = 29/Mar/2018  # Thursday
        >>> d + MO  # Equivalent to MO(1)
        BeautifulDate(2018, 4, 2)

        Get second to next Monday:
        >>> d = 29/Mar/2018
        >>> d + MO(2)
        BeautifulDate(2018, 4, 9)

        Get last Saturday:
        >>> d = 29/Mar/2018
        >>> d - SA
        BeautifulDate(2018, 3, 24)

        Get second to last Saturday:
        >>> d = 29/Mar/2018
        >>> d - SA(2)
        BeautifulDate(2018, 3, 17)

        Get second to last Saturday (same as previous):
        >>> d = 29/Mar/2018
        >>> d + SA(-2)
        BeautifulDate(2018, 3, 17)
    """

    def __init__(self, wd, n=1):
        self.wd = wd
        self.n = n

    def __radd__(self, other):
        return other + self.wd(self.n) * _weekday

    def __rsub__(self, other):
        return other + self.wd(-self.n) * _weekday

    def __call__(self, n):
        return BeautifulWeekday(self.wd, n)


weekdays = MO, TU, WE, TH, FR, SA, SU = [BeautifulWeekday(weekdays[i]) for i in range(7)]



from datetime import date, datetime

from beautiful_date import BeautifulDate, days


def timedelta_is_negative(td):
    """Checks whether timedelta is negative (would move date/datetime to the past)."""
    return datetime.now() > datetime.now() + td


class drange:
    """
    drange(stop) -> drange object
    drange(start, stop[, step]) -> drange object

    Return an object that produces a sequence of dates/datetimes (depending on parameter type)
    from start (inclusive) to stop (exclusive) by step.

    drange(stop) produces range of dates/datetimes from now to given stop date/datetime.
    drange(start, stop[, step]) produces range of dates/datetimes from start to stop.

    When step is given, it specifies the increment (or decrement).
    When step is not given, 1-day step is used.
    """

    def __init__(self, start_or_stop, stop=None, step=1 * days):
        if stop is None:
            if isinstance(start_or_stop, date):
                now = date.today()
            else:
                now = datetime.now()
            start = BeautifulDate(now.year, now.month, now.day)
            stop = start_or_stop
        else:
            start = start_or_stop

        if not step:
            raise ValueError('drange() step must be positive or negative step, not 0')

        self._backwards = timedelta_is_negative(step)

        self._start = start
        self._stop = stop
        self._step = step

    def __repr__(self):
        return 'drange({}, {}, {})'.format(self._start, self._stop, self._step)

    def __iter__(self):
        return self

    def __next__(self):
        if (self._backwards and self._start <= self._stop) or (not self._backwards and self._start >= self._stop):
            raise StopIteration

        ret = self._start
        self._start += self._step
        return ret



from datetime import date, datetime


class BeautifulDate(date):
    """Date object that can be extended to datetime by using Python indexing/slicing:

    Examples:
        >>> (Oct / 16 / 1995)[:]
        datetime.datetime(1995, 10, 16, 0, 0)

        >>> (Oct / 16 / 1995)[23]
        datetime.datetime(1995, 10, 16, 23, 0)

        >>> (Oct / 16 / 1995)[23:14]
        datetime.datetime(1995, 10, 16, 23, 14)

        >>> (Oct / 16 / 1995)[23:14:10]
        datetime.datetime(1995, 10, 16, 23, 14, 10)
    """

    def __getitem__(self, t):
        """
        Converts date to datetime with provided time [hours[:minutes[:seconds]]]
        :return: datetime object.
        """

        if isinstance(t, slice):
            h, m, s = t.start or 0, t.stop or 0, t.step or 0
        elif isinstance(t, int):
            h, m, s = t, 0, 0
        else:
            raise TypeError("Time values must be integer or slice, not {!r}".format(t.__class__.__name__))

        return datetime(self.year, self.month, self.day, hour=h, minute=m, second=s)

    def to_date(self):
        """
        Converts BeautifulDate to a simple Python date
        :return: date object
        """
        return date(self.year, self.month, self.day)


# Classes to build date
#   D @ 16/10/1995 (16/Oct/1995)
#   D @ 5/19/2006 (May/19/2006)

class _PartialDate:
    """Date builder that uses operator "/" or "-" between values of day, month and year

    Examples:
        >>> D @ 11/12/2000
        BeautifulDate(2000, 12, 11)

        >>> D @ 22-10-2000
        BeautifulDate(2000, 10, 22)
    """

    def __init__(self, first, _format):
        self._date_values = [first]
        self._format = _format

    def __truediv__(self, value):
        self._date_values.append(value)
        if len(self._date_values) == 3:
            return BeautifulDate(**dict(zip(self._format, self._date_values)))
        else:
            return self

    __sub__ = __truediv__

    def __str__(self):
        return '/'.join(str(v) for v in self._date_values)

    def __repr__(self):
        return '{}({})'.format(self.__class__.__name__, self.__str__())


class BaseDateFormat:
    """Base class for date format.

    Used to create PartialDate with a format specified in the inherited classes.

    Examples:
        >>> D @ 11
        _PartialDate(11)

        >>> D @ 22/10
        _PartialDate(22/10)
    """

    # List of strings 'day', 'month', and 'year' in desired order.
    # Should be overridden in the inherited classes
    _format = None

    def __matmul__(self, first):
        return _PartialDate(first, self._format)

    def __str__(self):
        return '{}{}'.format(self.__class__.__name__, self._format)

    @staticmethod
    def today():
        today = date.today()
        return BeautifulDate(year=today.year, month=today.month, day=today.day)

    @staticmethod
    def now():
        return datetime.now()


class DMY(BaseDateFormat):
    _format = 'day', 'month', 'year'


class MDY(BaseDateFormat):
    _format = 'month', 'day', 'year'


class YMD(BaseDateFormat):
    _format = 'year', 'month', 'day'


class YDM(BaseDateFormat):
    _format = 'year', 'day', 'month'


D = DMY()


# Classes to build date with month name
#   16/Oct/1995
#   May-19-2006

class _Day:
    """Second step of creating date object

    Stores month and day numbers. If applied operator '/' or '-', returns BeautifulDate with provided value of the year

    Examples:
        >>> 16/Oct/1995
        BeautifulDate(1995, 10, 16)

        >>> May-19-2006
        BeautifulDate(2006, 5, 19)

    """

    def __init__(self, d, m):
        self.d = d
        self.m = m

    def __sub__(self, y):
        return BeautifulDate(year=y, month=self.m, day=self.d)

    def __repr__(self):
        return '{}({}, {})'.format(self.__class__.__name__, self.d, self.m)

    __truediv__ = __sub__


class _Month:
    """First step of creating date object

    Stores month number. If applied operator '/' or '-', returns _Day with provided value of the day.

    Examples:
        >>> 16/Oct
        _Day(16, 10)

        >>> May-19
        _Day(19, 5)
    """

    def __init__(self, m):
        self.m = m

    def __sub__(self, d):
        return _Day(d, self.m)

    __rtruediv__ = __rsub__ = __truediv__ = __sub__


M = _, Jan, Feb, Mar, Apr, May, Jun, Jul, Aug, Sept, Oct, Nov, Dec = [_Month(i) for i in range(13)]



import unittest
from datetime import date, datetime
from beautiful_date import D, MDY, M, Jan, Feb, May, Oct, Jun, BeautifulDate


class TestBeautifulDate(unittest.TestCase):

    def test_date_create_by_month_name(self):
        self.assertEqual(16 / Oct / 1995, date(day=16, month=10, year=1995))
        self.assertEqual(May / 19 / 2006, date(day=19, month=5, year=2006))
        self.assertEqual(29 / Feb / 1996, date(day=29, month=2, year=1996))

        self.assertEqual(4 - Jan - 1970, date(day=4, month=1, year=1970))
        self.assertEqual(Feb - 7 - 1973, date(day=7, month=2, year=1973))
        self.assertEqual(29 - Feb - 1992, date(day=29, month=2, year=1992))

    def test_date_create_by_month_number(self):
        self.assertEqual(D @ 21 / 12 / 1995, date(day=21, month=12, year=1995))
        self.assertEqual(D @ 21 - 12 - 1994, date(day=21, month=12, year=1994))
        self.assertEqual(D @ 29 / 2 / 1988, date(day=29, month=2, year=1988))

        self.assertEqual(21 / M[12] / 1995, date(day=21, month=12, year=1995))
        self.assertEqual(21 - M[12] - 1994, date(day=21, month=12, year=1994))
        self.assertEqual(29 / M[2] / 1988, date(day=29, month=2, year=1988))

    def test_date_create_by_prefix_MDY(self):
        D = MDY()
        self.assertEqual(D @ 9 - 23 - 1996, date(day=23, month=9, year=1996))
        self.assertEqual(D @ 10 / 31 / 1995, date(day=31, month=10, year=1995))
        self.assertEqual(D @ 2 - 29 - 1984, date(day=29, month=2, year=1984))

    def test_datetime_create_by_month_name(self):
        self.assertEqual((30 / Jan / 1996)[23], datetime(day=30, month=1, year=1996, hour=23, minute=0))
        self.assertEqual((Feb - 23 - 1975)[1:22], datetime(day=23, month=2, year=1975, hour=1, minute=22))
        self.assertEqual((29 / Feb / 1992)[23:59:59],
                         datetime(day=29, month=2, year=1992, hour=23, minute=59, second=59))

    def test_datetime_create_by_prefix(self):
        self.assertEqual((D @ 8 - 6 - 1947)[5], datetime(day=8, month=6, year=1947, hour=5, minute=0))
        self.assertEqual((D @ 22 / 7 / 1938)[20:11], datetime(day=22, month=7, year=1938, hour=20, minute=11))
        self.assertEqual((D @ 29 - 2 - 1988)[10:11:12],
                         datetime(day=29, month=2, year=1988, hour=10, minute=11, second=12))

    def test_today_now(self):
        self.assertIsInstance(D.today(), BeautifulDate)
        self.assertIsInstance(D.now(), datetime)

    def test_time_type_error(self):
        with self.assertRaises(TypeError):
            _ = (15 / Jan / 2020)['evening']

    def test_to_date(self):
        d = (14 / Jun / 2022).to_date()
        self.assertIsInstance(d, date)
        self.assertEqual(d, date(2022, 6, 14))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import io
import os
import sys
from shutil import rmtree

from setuptools import setup, Command

here = os.path.abspath(os.path.dirname(__file__))


class UploadCommand(Command):
    """Support setup.py upload."""

    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(here, 'dist'))
        except OSError:
            pass

        self.status('Building Source and Wheel (universal) distribution…')
        os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status('Uploading the package to PyPi via Twine…')
        os.system('twine upload dist/*')

        self.status('Pushing git tags…')
        os.system('git tag v{0}'.format(VERSION))
        os.system('git push --tags')

        sys.exit()


NAME = 'beautiful-date'
DESCRIPTION = 'Simple and beautiful way to create date and datetime objects in Python.'
URL = 'https://github.com/kuzmoyev/beautiful-date'
DOWNLOAD_URL = 'https://github.com/kuzmoyev/beautiful-date/archive/1.0.tar.gz'
EMAIL = 'kuzmovych.goog@gmail.com'
AUTHOR = 'Yevhen Kuzmovych'
REQUIRES_PYTHON = '>=3.5.0'
VERSION = '2.1.0'

REQUIRED = [
    'python-dateutil>=2.7.0',
    'six>=1.11.0'
]
TESTS_REQUIRED = [
    "pytest>=5.4",
    "pytest-cov>=2.10",
    "flake8>3.8.3"
]

with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = '\n' + f.read()

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    download_url=DOWNLOAD_URL,
    py_modules=['beautiful_date', 'beautiful_timedelta', 'date_range'],
    packages=['beautiful_date'],
    install_requires=REQUIRED,
    tests_require=TESTS_REQUIRED,
    include_package_data=True,
    license='MIT',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    keywords=['beautiful', 'date', 'simple', 'timedelta', 'date-range'],
    cmdclass={
        'upload': UploadCommand,
    },
)



import unittest
from datetime import date, datetime

from beautiful_date import Feb, Mar, Oct, \
    years, months, weeks, days, hours, minutes, seconds, microseconds, leapday, \
    year, month, day, hour, minute, second, microsecond, yearday, nlyearday, \
    MO, SA


class TestBeautifulTimedelta(unittest.TestCase):

    def test_beautiful_delta_dates(self):
        d = 5 / Oct / 1995

        self.assertEqual(d + 5 * days, date(day=10, month=10, year=1995))
        self.assertEqual(d - 10 * days, date(day=25, month=9, year=1995))
        self.assertEqual(d + 27 * days, date(day=1, month=11, year=1995))
        self.assertEqual(d + 0.5 * days, date(day=5, month=10, year=1995))

        self.assertEqual(d + 3 * weeks, date(day=26, month=10, year=1995))
        self.assertEqual(d - 2 * weeks, date(day=21, month=9, year=1995))
        self.assertEqual(d + 7 * weeks, date(day=23, month=11, year=1995))
        self.assertEqual(d + 3 / 7 * weeks, date(day=8, month=10, year=1995))

        self.assertEqual(d + 2 * months, date(day=5, month=12, year=1995))
        self.assertEqual(d - 4 * months, date(day=5, month=6, year=1995))
        self.assertEqual(d + 5 * months, date(day=5, month=3, year=1996))

        self.assertEqual(d + 3 * years, date(day=5, month=10, year=1998))
        self.assertEqual(d + 10 * years, date(day=5, month=10, year=2005))

        self.assertEqual(d + 2 * days + 10 * months - 3 * years, date(day=7, month=8, year=1993))

        self.assertEqual((25 / Feb / 1996) + 10 * days + leapday, date(day=7, month=3, year=1996))
        self.assertEqual((25 / Feb / 1996) + 255 * yearday, date(day=11, month=9, year=1996))
        self.assertEqual((25 / Feb / 1996) + 255 * nlyearday, date(day=12, month=9, year=1996))
        self.assertEqual((25 / Feb / 1995) + 255 * yearday, date(day=12, month=9, year=1995))
        self.assertEqual((25 / Feb / 1995) + 255 * nlyearday, date(day=12, month=9, year=1995))

    def test_beautiful_delta_dates_inexact(self):
        last_day_of_month = 31 / Oct / 1995
        self.assertEqual(last_day_of_month + 1 * months, date(day=30, month=11, year=1995))
        self.assertEqual(last_day_of_month + 4 * months, date(day=29, month=2, year=1996))
        self.assertEqual(last_day_of_month + 4 * months + 1 * years, date(day=28, month=2, year=1997))

    def test_beautiful_delta_datetimes(self):
        dt = (16 / Oct / 1995)[1:2:3]

        self.assertEqual(dt + 25 * microseconds,
                         datetime(day=16, month=10, year=1995, hour=1, minute=2, second=3, microsecond=25))
        self.assertEqual(dt - 6 * 10 ** 5 * microseconds,
                         datetime(day=16, month=10, year=1995, hour=1, minute=2, second=2, microsecond=4 * 10 ** 5))
        self.assertEqual(dt + 10 ** 6 * microseconds,
                         datetime(day=16, month=10, year=1995, hour=1, minute=2, second=4))

        self.assertEqual(dt + 25 * seconds, datetime(day=16, month=10, year=1995, hour=1, minute=2, second=28))
        self.assertEqual(dt + 65 * seconds, datetime(day=16, month=10, year=1995, hour=1, minute=3, second=8))
        self.assertEqual(dt - 50 * seconds, datetime(day=16, month=10, year=1995, hour=1, minute=1, second=13))
        self.assertEqual(dt + 0.5 * seconds,
                         datetime(day=16, month=10, year=1995, hour=1, minute=2, second=3, microsecond=5 * 10 ** 5))

        self.assertEqual(dt + 13 * minutes, datetime(day=16, month=10, year=1995, hour=1, minute=15, second=3))
        self.assertEqual(dt + 65 * minutes, datetime(day=16, month=10, year=1995, hour=2, minute=7, second=3))
        self.assertEqual(dt - 20 * minutes, datetime(day=16, month=10, year=1995, hour=0, minute=42, second=3))
        self.assertEqual(dt + 1 / 6 * minutes, datetime(day=16, month=10, year=1995, hour=1, minute=2, second=13))

        self.assertEqual(dt + 2 * hours, datetime(day=16, month=10, year=1995, hour=3, minute=2, second=3))
        self.assertEqual(dt + 30 * hours, datetime(day=17, month=10, year=1995, hour=7, minute=2, second=3))
        self.assertEqual(dt - 13 * hours, datetime(day=15, month=10, year=1995, hour=12, minute=2, second=3))
        self.assertEqual(dt + 1 / 4 * hours, datetime(day=16, month=10, year=1995, hour=1, minute=17, second=3))

        self.assertEqual(dt + 3 * hours + 5 * minutes - 25 * seconds,
                         datetime(day=16, month=10, year=1995, hour=4, minute=6, second=38))

        dt += 3 * hours + 5 * minutes - 25 * seconds
        self.assertEqual(dt, datetime(day=16, month=10, year=1995, hour=4, minute=6, second=38))

    def test_beautiful_setter(self):
        dt = (16 / Oct / 1995)[1:2:3]

        self.assertEqual(dt + 5 * day, datetime(day=5, month=10, year=1995, hour=1, minute=2, second=3))
        self.assertEqual(dt + 3 * month, datetime(day=16, month=3, year=1995, hour=1, minute=2, second=3))
        self.assertEqual(dt + 1998 * year, datetime(day=16, month=10, year=1998, hour=1, minute=2, second=3))

        self.assertEqual(dt + 25 * microsecond,
                         datetime(day=16, month=10, year=1995, hour=1, minute=2, second=3, microsecond=25))
        self.assertEqual(dt + 55 * second, datetime(day=16, month=10, year=1995, hour=1, minute=2, second=55))
        self.assertEqual(dt + 56 * minute, datetime(day=16, month=10, year=1995, hour=1, minute=56, second=3))
        self.assertEqual(dt + 22 * hour, datetime(day=16, month=10, year=1995, hour=22, minute=2, second=3))

        self.assertEqual(dt + 1919 * year + 5 * month + 4 * day + 23 * hour + 41 * minute + 34 * second,
                         datetime(day=4, month=5, year=1919, hour=23, minute=41, second=34))

        dt += 1919 * year + 5 * month + 4 * day + 23 * hour + 41 * minute + 34 * second
        self.assertEqual(dt, datetime(day=4, month=5, year=1919, hour=23, minute=41, second=34))

    def test_beautiful_weekdays(self):
        d = 29 / Mar / 2018

        self.assertEqual(d + MO, date(2018, 4, 2))
        self.assertEqual(d + MO(2), date(2018, 4, 9))
        self.assertEqual(d - SA(2), date(2018, 3, 17))
        self.assertEqual(d + SA(-2), date(2018, 3, 17))
